package com.nisum.webflux_poc.controller;

import com.nisum.webflux_poc.dto.EmployeeDto;
import com.nisum.webflux_poc.repository.EmployeeRepository;
import com.nisum.webflux_poc.service.EmployeeService;
import com.nisum.webflux_poc.service.KafkaProducerService;
import com.nisum.webflux_poc.utills.AppUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.autoconfigure.web.reactive.WebFluxTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.test.util.ReflectionTestUtils;
import org.springframework.test.web.reactive.server.WebTestClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.io.IOException;
import java.util.List;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.when;

@ExtendWith(MockitoExtension.class)
@WebFluxTest(EmployeeController.class)
public class EmployeeControllerTest {



        @MockBean
        EmployeeService employeeService;

        @MockBean
        EmployeeRepository employeeRepository;

        @Autowired
        private WebTestClient webTestClient;

        @MockBean
        KafkaProducerService kafkaProducerService;

        @Value("${spring.kafka.template.default-topic}")
        private String topic;

        List<EmployeeDto> employeeList;

        private EmployeeDto employee;


        @BeforeEach
        void setUp() throws IOException {
            employeeList = AppUtils.readFile("static/request/EmployeeList.json");
            employee = AppUtils.readEmployeeFile("static/request/Employee.json");
            ReflectionTestUtils.setField(kafkaProducerService,"topic",topic);

        }

        @Test
        void getEmployeeDataTest() throws IOException {

            when(employeeService.getAllEmployees()).thenReturn(Flux.fromIterable(employeeList));
            webTestClient.get()
                    .uri("/employee")
                    .exchange()
                    .expectStatus()
                    .is2xxSuccessful()
                    .expectBodyList(EmployeeDto.class);
        }

    @Test
    void testGetEmployee_NoContent() {
        when(employeeService.getAllEmployees())
                .thenReturn(Flux.empty()); // Simulates failure to save

        webTestClient.get()
                .uri("/employee")
                .exchange()
                .expectStatus().isOk()
                .expectBodyList(EmployeeDto.class)
                .hasSize(0);
    }

    @Test
    void testGetEmployee_InternalServerError() {
        when(employeeService.getAllEmployees())
                .thenReturn(Flux.error(new RuntimeException("Database error")));

        webTestClient.get()
                .uri("/employee")
                .exchange()
                .expectStatus().isEqualTo(HttpStatus.INTERNAL_SERVER_ERROR);
    }


    @Test
    void testSaveEmployee_BadRequest() {
        when(employeeService.saveEmployeeData(any(Mono.class)))
                .thenReturn(Mono.empty()); // Simulates failure to save

        webTestClient.post()
                .uri("/employee/save")
                .contentType(MediaType.APPLICATION_JSON)
                .bodyValue(employee)
                .exchange()
                .expectStatus().isBadRequest();
    }

    @Test
    void testSaveEmployee_InternalServerError() {
        when(employeeService.saveEmployeeData(any(Mono.class)))
                .thenReturn(Mono.error(new RuntimeException("Database error"))); // Simulate error

        webTestClient.post()
                .uri("/employee/save")
                .contentType(MediaType.APPLICATION_JSON)
                .bodyValue(employee)
                .exchange()
                .expectStatus().isEqualTo(HttpStatus.INTERNAL_SERVER_ERROR);
    }


        @Test
    void saveEmployeeTest(){


            when(employeeService.saveEmployeeData(any(Mono.class))).thenReturn(Mono.just(employee));
            webTestClient.post()
                    .uri("/employee/save")
                    .contentType(MediaType.APPLICATION_JSON)
                    .bodyValue(employee)
                    .exchange()
                    .expectStatus()
                    .is2xxSuccessful()
                    .expectBodyList(EmployeeDto.class);
        }

    @Test
    void updateEmployeeTest(){
        String employeeId="1";
        Mockito.when(employeeService.getEmployee(any(String.class))).thenReturn(Mono.just(employee));
        webTestClient.put()
                .uri("/employee/update/{id}", employeeId)
                .contentType(MediaType.APPLICATION_JSON)
                .bodyValue(employee)
                .exchange()
                .expectStatus()
                .is2xxSuccessful()
                .expectBodyList(EmployeeDto.class);
    }

    @Test
    void testUpdateEmployee_NotFound(){
        String employeeId="1";
        Mockito.when(employeeService.getEmployee(any(String.class))).thenReturn(Mono.empty());
        webTestClient.put()
                .uri("/employee/update/{id}", employeeId)
                .contentType(MediaType.APPLICATION_JSON)
                .bodyValue(employee)
                .exchange()
                .expectStatus().isOk()
                .expectBody(Void.class);
    }

    @Test
    void testUpdateEmployee_BadRequest(){
        String employeeId="1";
        Mockito.when(employeeService.getEmployee(any(String.class)))
                .thenReturn(Mono.error(new RuntimeException("Bad request")));
        webTestClient.put()
                .uri("/employee/update/{id}", employeeId)
                .contentType(MediaType.APPLICATION_JSON)
                .exchange()
                .expectStatus().isEqualTo(HttpStatus.BAD_REQUEST);
    }
    @Test
    void getEmployeeWithIdTest(){
        String employeeId="1";
        Mono<EmployeeDto> employeeDto= Mono.just(employee);
        when(employeeService.getEmployee(employeeId)).thenReturn(employeeDto);
        webTestClient.get()
                .uri("/employee/{id}", employeeId)
                .exchange()
                .expectStatus()
                .is2xxSuccessful()
                .expectBodyList(EmployeeDto.class);
    }

    @Test
    void testDeleteEmployee_NotFound() {
        String employeeId = "123";

        when(employeeService.deleteEmployee(employeeId))
                .thenReturn(Mono.error(new RuntimeException("Database error"))); // Simulate error

        webTestClient.delete()
                .uri("/employee/delete/{id}", employeeId)
                .exchange()
                .expectStatus().isEqualTo(HttpStatus.INTERNAL_SERVER_ERROR);
    }


    @Test
    void deleteEmployeeTest(){
        String employeeId="1";
        when(employeeService.deleteEmployee(any(String.class))).thenReturn(Mono.empty());
        webTestClient.delete()
                .uri("/employee/delete/{id}", employeeId)
                .exchange()
                .expectStatus()
                .is2xxSuccessful()
                .expectBodyList(EmployeeDto.class);
    }

    @Test
    void getAverageSalaryTest(){
        Double averageSalary= 90.9;
        Mockito.when(employeeService.getAverageSalary()).thenReturn(Flux.just(averageSalary));
        webTestClient.get()
                .uri("/employee/average/salary")
                .exchange()
                .expectStatus()
                .is2xxSuccessful()
                .expectBodyList(Double.class);
    }

    @Test
    void getAverageSalaryTestWithException(){
        Double averageSalary= 90.9;
        Mockito.when(employeeService.getAverageSalary()).thenReturn(Flux.error(new RuntimeException("internal server error")));
        webTestClient.get()
                .uri("/employee/average/salary")
                .exchange()
                .expectStatus()
                .isEqualTo(HttpStatus.INTERNAL_SERVER_ERROR);
    }

    @Test
    void sendDataToKafkaTopicTest(){

            String message = "manasa";
        Mockito.when(kafkaProducerService.sendMessage(anyString())).thenReturn("test");
        webTestClient.get()
                .uri("/employee/kafka/send/{message}", message)
                .exchange()
                .expectStatus()
                .is2xxSuccessful();
    }

    @Test
    void sendDataToKafkaTopicWithException(){

        String message = "manasa";
        Mockito.when(kafkaProducerService.sendMessage(anyString())).thenThrow(new RuntimeException("internal server error"));
        webTestClient.get()
                .uri("/employee/kafka/send/{message}", message)
                .exchange()
                .expectStatus()
                .isEqualTo(HttpStatus.INTERNAL_SERVER_ERROR);
    }


}
